/*************************
Diese Software ist ein Beispiel (sample code) und unterliegt der Microsoft Public License. 
Die Verwendung des Codes ist unter den Bedingungen der Microsoft Public License erlaubt.
*************************
This software is sample code and is subject to the Microsoft Public License. 
You may use this code according to the conditions of the Microsoft Public License.
*************************/

using System;
using System.Collections.Specialized;
using System.Diagnostics;
using System.IO;
using System.Net;
using System.Text;
using System.Web;
using System.Web.UI;
using Egora.Stammportal.HttpReverseProxy.Properties;
using Egora.Stammportal.HttpReverseProxy.StreamFilter;

namespace Egora.Stammportal.HttpReverseProxy
{
  public class HttpReverseProxyHandler : IHttpHandler //, IHttpAsyncHandler // for later
  {
    private static bool s_initialized;
    private static object s_initializeLock = new object();
    private static Egora.Stammportal.HttpReverseProxy.Mapping.PathMap s_map;
    public const string LogFileDirKey = "C8411A94-7870-473A-983F-48D77C940C54";

    public enum Event
    {
      Initializing = 100,
      ProcessingRequest,
      ProcessingResponse,
      NotFound,
      RetrievingApplications,
      NotAuthorized,
      StreamFilter,
    }

    protected virtual void Initialize()
    {
      TraceHelper.Trace.TraceEvent(TraceEventType.Start, (int) Event.Initializing,
                                   "ReverseProxy initializing started.");

      string filename =
        HttpContext.Current.Server.MapPath(
          Egora.Stammportal.HttpReverseProxy.Properties.Settings.Default.PathMapFile);
      lock (s_initializeLock)
      {
        s_map = Egora.Stammportal.HttpReverseProxy.Mapping.PathMap.CreateFromFile(filename);
        RemoteApplication.CreateRemoteApplications(s_map);

        ServicePointManager.MaxServicePointIdleTime = Settings.Default.ConnectionMaxIdleTimeSeconds*1000;
        ServicePointManager.DefaultConnectionLimit = Settings.Default.ConnectionsPerServer;

        s_initialized = true;
      }

      TraceHelper.Trace.TraceEvent(TraceEventType.Start, (int) Event.Initializing,
                                   "ReverseProxy initializing finished.");
    }

    #region IHttpHandler Members

    bool IHttpHandler.IsReusable
    {
      get { return false; }
    }

    void IHttpHandler.ProcessRequest(HttpContext context)
    {
      TraceHelper.Trace.TraceEvent(TraceEventType.Verbose, (int) Event.ProcessingRequest,
                                   "Processing request started.");
      if (context.Request == null)
      {
        //happens when application starts first time
        TraceHelper.Trace.TraceEvent(TraceEventType.Verbose, (int) Event.ProcessingRequest, "Context is null.");
        return;
      }

      if (!s_initialized)
        Initialize();

      Authentication auth = new Authentication(context.Request);
      string path = context.Request.Path.ToLowerInvariant();
      string adminpath = (context.Request.ApplicationPath + Settings.Default.AdministrationPath + "/").ToLowerInvariant();
      if (path.StartsWith(adminpath))
      {
        if (auth.IsAdmin)
        {
          if (path.Replace(adminpath, "") == "reset.aspx")
            Initialize();
          string pagePath = path.Replace(adminpath, context.Request.ApplicationPath + "/Administration/");
          IHttpHandler pageHandler =
            PageParser.GetCompiledPageInstance(path, context.Request.MapPath(pagePath), context);
          pageHandler.ProcessRequest(context);
          return;
        }
      }

      RemoteApplication remoteApplication = RemoteApplication.GetRemoteApplication(context.Request);

      if (remoteApplication == null)
      {
        TraceHelper.Trace.TraceEvent(TraceEventType.Warning, (int) Event.NotFound,
                                     "No RemoteApplication found. Ending with 404.");
        context.Response.StatusCode = 404;
        context.Response.End();
      }

      StringBuilder p = new StringBuilder(remoteApplication.RemoteApplicationProxyPath);
      p.Replace('/', '_');
      foreach (char i in Path.GetInvalidPathChars())
        p.Replace(i, '_');

      _logFileDir = Path.Combine(Path.Combine(Settings.Default.TrafficLogDir, p.ToString()),
                           DateTime.Now.ToString("yyyyMMdd_HHmmss") + "_" +
                           Guid.NewGuid());
      context.Items.Add(LogFileDirKey, _logFileDir);

      HttpWebRequest webRequest = null;
      HttpWebResponse webResponse = null;

      using (Stream inputBuffer = RequestHandler.GetInputStream(context.Request))
      {
        for (int tries = 0; tries <= Properties.Settings.Default.NetworkRetryCount; tries++)
        {
          if (tries > 0)
          {
            System.Threading.Thread.Sleep(Properties.Settings.Default.NetworkRetryDelay);
            TraceHelper.Trace.TraceEvent(TraceEventType.Error, (int) Event.ProcessingResponse,
                                         "trying again.");
            inputBuffer.Position = 0;
          }

          try
          {
            webRequest = remoteApplication.CreateRightSideRequest(context.Request, inputBuffer);
            
            if (remoteApplication.LogTraffic)
              LogRequestInformation(webRequest);
          }
          catch (AuthorizationException e)
          {
            TraceHelper.Trace.TraceEvent(TraceEventType.Verbose, (int) Event.NotAuthorized, e.Message);
            context.Response.StatusCode = 406;
            context.Response.End();
          }

          TraceHelper.Trace.TraceEvent(TraceEventType.Verbose, (int) Event.ProcessingRequest,
                                       "Processing request finished.");

          try
          {
            webResponse = auth.GetWebResponse(webRequest);
          }
          catch (WebException ex)
          {
            TraceHelper.Trace.TraceEvent(TraceEventType.Error, (int) Event.ProcessingResponse,
                                         "Could not get right side response: {0}", ex.Message);
            if (ex.Response == null)
            {
              if (IsRetryable(ex, remoteApplication.IsSoap))
              {
                if (webRequest != null)
                {
                  webRequest.Abort();
                  webRequest = null;
                }
                continue;
              }
              else if (ex.Status == WebExceptionStatus.Timeout)
              {
                TraceHelper.Trace.TraceEvent(TraceEventType.Error, (int) Event.ProcessingResponse,
                                             "Ending with 408 Timeout.");
                context.Response.StatusCode = 408;
                context.Response.End();
                /*
                408 Request Timeout Der Server hat eine erwartete Anfrage nicht innerhalb des dafr festgelegten Maximalzeitraums erhalten. Die Verbindung zum anfragenden Browser wird deshalb abgebaut. Angeforderte Daten werden nicht bertragen. 
                412 Precondition Failed Eine oder mehrere Bedingungen, die bei der Anfrage gestellt wurden, treffen nicht zu. Die angeforderten Daten werden deshalb nicht bertragen. 
                500 Internal Server Error Der Server kann die angeforderten Daten nicht senden, weil auf dem Server ein Fehler aufgetreten ist. Beispielsweise konnte das aufgerufene CGI-Script nicht gestartet werden. 
                502 Bad Gateway Zum Bearbeiten der Anfrage musste der Server einen anderen Server aufrufen, erhielt dabei jedoch eine Fehlermeldung. Die angeforderten Daten knnen deshalb nicht gesendet werden. 
                503 Service Unavailable Der Server kann die Anfrage wegen berlastung nicht bearbeiten. Die angeforderten Daten knnen deshalb nicht gesendet werden. In der Statusmeldung kann stehen, wann die Anfrage frhestens wieder bearbeitet werden kann. Im Gegensatz zum Status-Code 202 verarbeitet der Server die Daten nicht, sobald er wieder Kapazitten hat. 
                504 Gateway Timeout Zum Bearbeiten der Anfrage musste der Server einen anderen Server aufrufen, erhielt dabei jedoch nach einem festgelegten Maximalzeitraum keine Antwort. Die angeforderten Daten knnen deshalb nicht gesendet werden. 
                */
              }
              else
              {
                TraceHelper.Trace.TraceEvent(TraceEventType.Error, (int) Event.ProcessingResponse,
                                             "Ending with 500.");

                TraceHelper.Trace.TraceEvent(TraceEventType.Error, (int) Event.ProcessingResponse,
                                             "Input stream follows.");
                try
                {
                  inputBuffer.Position = 0;
                  byte[] buffer = new byte[inputBuffer.Length];
                  inputBuffer.Read(buffer, 0, (int) inputBuffer.Length);
                  Encoding encoding = Encoding.UTF8;
                  if (context.Request.ContentEncoding != null)
                    encoding = context.Request.ContentEncoding;
                  TraceHelper.Trace.TraceData(TraceEventType.Error, (int) Event.ProcessingResponse,
                                              encoding.GetString(buffer));
                }
                catch
                {
                }
                context.Response.StatusCode = 500;
                context.Response.StatusDescription = ex.Message;
                context.Response.End();
              }
            }
            else
            {
              webResponse = (HttpWebResponse) ex.Response;
            }
          } // end catch webResponse = auth.GetWebResponse (webRequest);

          if (ShouldRetry(webResponse, remoteApplication.IsSoap))
          {
            TraceHelper.Trace.TraceEvent(TraceEventType.Error, (int) Event.ProcessingResponse,
                                         "Received " + webResponse.StatusCode);
            if (tries < Properties.Settings.Default.NetworkRetryCount)
              webResponse.Close();
            webRequest = null;
            continue;
          }
          break;
        } // end for tries
      } // end using inputBuffer

      if (webResponse == null)
        throw new ApplicationException("Response is null.");

      bool errorEncountered = webResponse.StatusCode.Equals(HttpStatusCode.InternalServerError);
      bool bufferResponse = Settings.Default.BufferRightSide || errorEncountered || remoteApplication.LogTraffic;
      using (Stream output = GetOutputStream(context.Response, webResponse.ContentLength, bufferResponse))
      {
        using (HttpWebResponse response = webResponse)
        {
          TraceHelper.Trace.TraceEvent(TraceEventType.Verbose, (int) Event.ProcessingResponse,
                                       "Processing response started.");
          if (remoteApplication.LogTraffic)
          {
            LogResponseInformation(response);
          } 
          
          remoteApplication.ShapeHttpResponse(response, context.Response);

          Stream rightSideResponseStream = response.GetResponseStream();
          CopyFilter filter = new CopyFilter(response.ContentLength);
          filter.FilterStream(rightSideResponseStream, output);

          TraceHelper.Trace.TraceEvent(TraceEventType.Verbose, (int) Event.ProcessingResponse,
                                       "Processing response finished.");
        }

        if (errorEncountered)
        {
          output.Position = 0;
          byte[] buffer = new byte[output.Length];
          output.Read(buffer, 0, (int) output.Length);
          Encoding encoding = Encoding.UTF8;
          if (context.Response.ContentEncoding != null)
            encoding = context.Response.ContentEncoding;
          TraceHelper.Trace.TraceEvent(TraceEventType.Error, (int) Event.ProcessingResponse,
                                       "Response stream follows.");
          TraceHelper.Trace.TraceData(TraceEventType.Error, (int) Event.ProcessingResponse,
                                      encoding.GetString(buffer));
        }

        if (remoteApplication.LogTraffic)
        {
          LogResponseContent((MemoryStream) output);
        }

        if (bufferResponse)
        {
          output.Position = 0;
          CopyFilter filter = new CopyFilter(output.Length);
          filter.FilterStream(output, context.Response.OutputStream);
        }

        context.Response.End();
      }
    }

    private void LogRequestInformation(HttpWebRequest webRequest)
    {
      string logFileName =  _logFileDir +"\\1_RequestHeaders.txt";
      NameValueCollection headers = new NameValueCollection(webRequest.Headers);
      headers.Add("Input-Headers", HttpContext.Current.Request.ServerVariables["ALL_RAW"]);
      LogHeader(headers, logFileName);
    }

    private void LogHeader(NameValueCollection webHeaderCollection, string logFileName)
    {
      try
      {
        EnsureLogDir();
        StringBuilder h = new StringBuilder();
        foreach (string key in webHeaderCollection.AllKeys)
        {
          h.Append(key).Append(": ").AppendLine(webHeaderCollection[key]);
        }
        File.WriteAllText(logFileName, h.ToString());
      }
      catch (Exception e)
      {
        TraceHelper.Trace.TraceEvent(TraceEventType.Error, (int) Event.ProcessingResponse, e.Message);
      }
    }

    private void LogResponseInformation(HttpWebResponse response)
    {
      string logFileName = _logFileDir + "\\3_ResponseHeaders.txt";
      NameValueCollection headers = new NameValueCollection(response.Headers);
      headers.Add("StatusCode", response.StatusCode.ToString("D"));
      headers.Add("StatusDescription", response.StatusDescription); 
      LogHeader(headers, logFileName);
    }

    private void LogResponseContent(MemoryStream output)
    {
      output.Position = 0;
      try
      {
        EnsureLogDir();
        File.WriteAllBytes(_logFileDir + "\\4_Response.txt", output.ToArray());
      }
      catch (Exception e)
      {
        TraceHelper.Trace.TraceEvent(TraceEventType.Error, (int) Event.ProcessingResponse, e.Message);
      }
      output.Position = 0;
    }

    private void EnsureLogDir()
    {
      if (!Directory.Exists(_logFileDir))
        Directory.CreateDirectory(_logFileDir);
    }

    private Stream GetOutputStream(HttpResponse response, long bufferSize, bool useMemoryStream)
    {
      Stream output;
      int size = bufferSize > 0 ? (int) bufferSize : 1024;
      if (useMemoryStream)
      {
        output = new MemoryStream(size);
      }
      else
      {
        output = response.OutputStream;
      }
      return output;
    }

    private static string[] s_retryableExceptionMessages =
      Properties.Settings.Default.RetryableErrorMessages.Split(';');

    private bool IsRetryable(WebException ex, bool isSoap)
    {
      if (isSoap && !Settings.Default.RetrySoap)
        return false;

      foreach (string m in s_retryableExceptionMessages)
      {
        if (ex.Message.Contains(m))
          return true;
      }
      return false;
    }

    private static string[] s_retryableHosts = Properties.Settings.Default.RetryableHosts.Split(';');
    private string _logFileDir;

    private bool ShouldRetry(HttpWebResponse webResponse, bool isSoap)
    {
      if (isSoap && !Settings.Default.RetrySoap)
        return false;

      if (webResponse == null)
        return false;

      bool retry = webResponse.StatusCode == HttpStatusCode.InternalServerError;

      if (!retry)
        return false;

      string url = webResponse.ResponseUri.AbsoluteUri.ToLowerInvariant();
      foreach (string host in s_retryableHosts)
      {
        if (url.Contains(host))
          return retry;
      }

      return false;
    }

    #endregion

    #region IHttpAsyncHandler Members

    //IAsyncResult IHttpAsyncHandler.BeginProcessRequest(HttpContext context, AsyncCallback cb, object extraData)
    //{
    //  throw new Exception("The method or operation is not implemented.");
    //}

    //void IHttpAsyncHandler.EndProcessRequest(IAsyncResult result)
    //{
    //  throw new Exception("The method or operation is not implemented.");
    //}

    #endregion
  }
}